@@ -17,8 +17,7 @@ local report_otf          = logs.reporter("fonts","otf loading")
 
 local fonts               = fonts
 local otf                 = fonts.handlers.otf
-local otffeatures         = fonts.constructors.newfeatures("otf")
-local registerotffeature  = otffeatures.register
+local registerotffeature  = otf.features.register
 local setmetatableindex   = table.setmetatableindex
 
 -- In the userdata interface we can not longer tweak the loaded font as
@@ -71,6 +70,7 @@ local function addfeature(data,feature,specifications)
                 local subtables     = specification.subtables or { specification.data } or { }
                 local featuretype   = types[specification.type or "substitution"]
                 local featureflags  = specification.flags or noflags
+                local featureorder  = specification.order or { feature }
                 local added         = false
                 local featurename   = format("ctx_%s_%s",feature,s)
                 local st = { }
@@ -134,14 +134,20 @@ local function addfeature(data,feature,specifications)
                             askedfeatures[k] = table.tohash(v)
                         end
                     end
-                    sequences[#sequences+1] = {
+                    local sequence = {
                         chain     = 0,
                         features  = { [feature] = askedfeatures },
                         flags     = featureflags,
                         name      = featurename,
+                        order     = featureorder,
                         subtables = st,
                         type      = featuretype,
                     }
+                    if specification.prepend then
+                        insert(sequences,1,sequence)
+                    else
+                        insert(sequences,sequence)
+                    end
                     -- register in metadata (merge as there can be a few)
                     if not gsubfeatures then
                         gsubfeatures  = { }
@@ -166,7 +172,7 @@ local function addfeature(data,feature,specifications)
             end
         end
         if trace_loading then
-            report_otf("enhance: registering feature '%s', %s glyphs affected, %s glyphs skipped",feature,done,skip)
+            report_otf("registering feature %a, affected glyphs %a, skipped glyphs %a",feature,done,skip)
         end
     end
 end
@@ -189,9 +195,11 @@ otf.enhancers.register("check extra features",enhance)
 
 -- tlig --
 
-local tlig = {
-    endash        = "hyphen hyphen",
-    emdash        = "hyphen hyphen hyphen",
+local tlig = { -- we need numbers for some fonts so ...
+ -- endash        = "hyphen hyphen",
+ -- emdash        = "hyphen hyphen hyphen",
+    [0x2013]      = { 0x002D, 0x002D },
+    [0x2014]      = { 0x002D, 0x002D, 0x002D },
  -- quotedblleft  = "quoteleft quoteleft",
  -- quotedblright = "quoteright quoteright",
  -- quotedblleft  = "grave grave",
@@ -203,7 +211,9 @@ local tlig_specification = {
     type     = "ligature",
     features = everywhere,
     data     = tlig,
+    order    = { "tlig" },
     flags    = noflags,
+    prepend  = true,
 }
 
 otf.addfeature("tlig",tlig_specification)
@@ -225,7 +235,9 @@ local trep_specification = {
     type      = "substitution",
     features  = everywhere,
     data      = trep,
+    order     = { "trep" },
     flags     = noflags,
+    prepend   = true,
 }
 
 otf.addfeature("trep",trep_specification)
@@ -255,6 +267,7 @@ if characters.combined then
         type       = "ligature",
         features   = everywhere,
         data       = tcom,
+        order     = { "tcom" },
         flags      = noflags,
         initialize = initialize,
     }
@@ -312,14 +325,16 @@ end
 local anum_specification = {
     {
         type     = "substitution",
-        features = { arab = { URD = true, dflt = true } },
+        features = { arab = { urd = true, dflt = true } },
+        order    = { "anum" },
         data     = anum_arabic,
         flags    = noflags, -- { },
         valid    = valid,
     },
     {
         type     = "substitution",
-        features = { arab = { URD = true } },
+        features = { arab = { urd = true } },
+        order    = { "anum" },
         data     = anum_persian,
         flags    = noflags, -- { },
         valid    = valid,
