@@ -9,6 +9,8 @@ if not modules then modules = { } end modules ['font-otp'] = {
 -- todo: pack math (but not that much to share)
 --
 -- pitfall 5.2: hashed tables can suddenly become indexed with nil slots
+--
+-- unless we sort all hashes we can get a different pack order (no big deal but size can differ)
 
 local next, type = next, type
 local sort, concat = table.sort, table.concat
@@ -140,6 +142,11 @@ end
 --     return b
 -- end
 
+-- beware: we cannot unpack and repack the same table because then sharing
+-- interferes (we could catch this if needed) .. so for now: save, reload
+-- and repack in such cases (never needed anyway) .. a tricky aspect is that
+-- we then need to sort more thanks to random hashing
+
 local function packdata(data)
     if data then
      -- stripdata(data)
@@ -750,27 +757,28 @@ local function unpackdata(data)
                                     rule.replacements = tv
                                 end
                             end
-                            local fore = rule.fore
-                            if fore then
-                                local tv = tables[fore]
-                                if tv then
-                                    rule.fore = tv
-                                end
-                            end
-                            local back = rule.back
-                            if back then
-                                local tv = tables[back]
-                                if tv then
-                                    rule.back = tv
-                                end
-                            end
-                            local names = rule.names
-                            if names then
-                                local tv = tables[names]
-                                if tv then
-                                    rule.names = tv
-                                end
-                            end
+                         -- local fore = rule.fore
+                         -- if fore then
+                         --     local tv = tables[fore]
+                         --     if tv then
+                         --         rule.fore = tv
+                         --     end
+                         -- end
+                         -- local back = rule.back
+                         -- if back then
+                         --     local tv = tables[back]
+                         --     if tv then
+                         --         rule.back = tv
+                         --     end
+                         -- end
+                         -- local names = rule.names
+                         -- if names then
+                         --     local tv = tables[names]
+                         --     if tv then
+                         --         rule.names = tv
+                         --     end
+                         -- end
+                            --
                             local lookups = rule.lookups
                             if lookups then
                                 local tv = tables[lookups]
@@ -897,3 +905,4 @@ if otf.enhancers.register then
 end
 
 otf.enhancers.unpack = unpackdata -- used elsewhere
+otf.enhancers.pack   = packdata   -- used elsewhere
