@@ -15,51 +15,53 @@ if not lfs then
     lfs = optionalrequire("lfs")
 end
 
-if not lfs then
-
-    lfs = {
-        getcurrentdir = function()
-            return "."
-        end,
-        attributes = function()
-            return nil
-        end,
-        isfile = function(name)
-            local f = io.open(name,'rb')
-            if f then
-                f:close()
-                return true
-            end
-        end,
-        isdir = function(name)
-            print("you need to load lfs")
-            return false
-        end
-    }
-
-elseif not lfs.isfile then
-
-    local attributes = lfs.attributes
-
-    function lfs.isdir(name)
-        return attributes(name,"mode") == "directory"
-    end
-
-    function lfs.isfile(name)
-        return attributes(name,"mode") == "file"
-    end
-
- -- function lfs.isdir(name)
- --     local a = attributes(name)
- --     return a and a.mode == "directory"
- -- end
-
- -- function lfs.isfile(name)
- --     local a = attributes(name)
- --     return a and a.mode == "file"
- -- end
-
-end
+-- -- see later
+--
+-- if not lfs then
+--
+--     lfs = {
+--         getcurrentdir = function()
+--             return "."
+--         end,
+--         attributes = function()
+--             return nil
+--         end,
+--         isfile = function(name)
+--             local f = io.open(name,'rb')
+--             if f then
+--                 f:close()
+--                 return true
+--             end
+--         end,
+--         isdir = function(name)
+--             print("you need to load lfs")
+--             return false
+--         end
+--     }
+--
+-- elseif not lfs.isfile then
+--
+--     local attributes = lfs.attributes
+--
+--     function lfs.isdir(name)
+--         return attributes(name,"mode") == "directory"
+--     end
+--
+--     function lfs.isfile(name)
+--         return attributes(name,"mode") == "file"
+--     end
+--
+--  -- function lfs.isdir(name)
+--  --     local a = attributes(name)
+--  --     return a and a.mode == "directory"
+--  -- end
+--
+--  -- function lfs.isfile(name)
+--  --     local a = attributes(name)
+--  --     return a and a.mode == "file"
+--  -- end
+--
+-- end
 
 local insert, concat = table.insert, table.concat
 local match, find, gmatch = string.match, string.find, string.gmatch
@@ -72,6 +74,28 @@ local checkedsplit = string.checkedsplit
 
 local P, R, S, C, Cs, Cp, Cc, Ct = lpeg.P, lpeg.R, lpeg.S, lpeg.C, lpeg.Cs, lpeg.Cp, lpeg.Cc, lpeg.Ct
 
+-- better this way:
+
+local tricky     = S("/\\") * P(-1)
+local attributes = lfs.attributes
+
+if sandbox then
+    sandbox.redefine(lfs.isfile,"lfs.isfile")
+    sandbox.redefine(lfs.isdir, "lfs.isdir")
+end
+
+function lfs.isdir(name)
+    if lpegmatch(tricky,name) then
+        return attributes(name,"mode") == "directory"
+    else
+        return attributes(name.."/.","mode") == "directory"
+    end
+end
+
+function lfs.isfile(name)
+    return attributes(name,"mode") == "file"
+end
+
 local colon     = P(":")
 local period    = P(".")
 local periods   = P("..")
@@ -133,8 +157,8 @@ file.suffix       = suffixonly
 file.suffixesonly = suffixesonly
 file.suffixes     = suffixesonly
 
-file.dirname    = pathpart   -- obsolete
-file.extname    = suffixonly -- obsolete
+file.dirname      = pathpart   -- obsolete
+file.extname      = suffixonly -- obsolete
 
 -- actually these are schemes
 
@@ -368,11 +392,14 @@ function file.joinpath(tab,separator) -- table
     return tab and concat(tab,separator or io.pathseparator) -- can have trailing //
 end
 
+local someslash = S("\\/")
 local stripper  = Cs(P(fwslash)^0/"" * reslasher)
-local isnetwork = fwslash * fwslash * (1-fwslash) + (1-fwslash-colon)^1 * colon
+local isnetwork = someslash * someslash * (1-someslash)
+                + (1-fwslash-colon)^1 * colon
 local isroot    = fwslash^1 * -1
 local hasroot   = fwslash^1
 
+local reslasher = lpeg.replacer(S("\\/"),"/")
 local deslasher = lpeg.replacer(S("\\/")^1,"/")
 
 -- If we have a network or prefix then there is a change that we end up with two
@@ -382,26 +409,90 @@ local deslasher = lpeg.replacer(S("\\/")^1,"/")
 -- then we still have to deal with urls ... anyhow, multiple // are never a real
 -- problem but just ugly.
 
-function file.join(...)
-    local lst = { ... }
-    local one = lst[1]
+-- function file.join(...)
+--     local lst = { ... }
+--     local one = lst[1]
+--     if lpegmatch(isnetwork,one) then
+--         local one = lpegmatch(reslasher,one)
+--         local two = lpegmatch(deslasher,concat(lst,"/",2))
+--         if lpegmatch(hasroot,two) then
+--             return one .. two
+--         else
+--             return one .. "/" .. two
+--         end
+--     elseif lpegmatch(isroot,one) then
+--         local two = lpegmatch(deslasher,concat(lst,"/",2))
+--         if lpegmatch(hasroot,two) then
+--             return two
+--         else
+--             return "/" .. two
+--         end
+--     elseif one == "" then
+--         return lpegmatch(stripper,concat(lst,"/",2))
+--     else
+--         return lpegmatch(deslasher,concat(lst,"/"))
+--     end
+-- end
+
+function file.join(one, two, three, ...)
+    if not two then
+        return one == "" and one or lpegmatch(stripper,one)
+    end
+    if one == "" then
+        return lpegmatch(stripper,three and concat({ two, three, ... },"/") or two)
+    end
     if lpegmatch(isnetwork,one) then
-        local two = lpegmatch(deslasher,concat(lst,"/",2))
-        return one .. "/" .. two
+        local one = lpegmatch(reslasher,one)
+        local two = lpegmatch(deslasher,three and concat({ two, three, ... },"/") or two)
+        if lpegmatch(hasroot,two) then
+            return one .. two
+        else
+            return one .. "/" .. two
+        end
     elseif lpegmatch(isroot,one) then
-        local two = lpegmatch(deslasher,concat(lst,"/",2))
+        local two = lpegmatch(deslasher,three and concat({ two, three, ... },"/") or two)
         if lpegmatch(hasroot,two) then
             return two
         else
             return "/" .. two
         end
-    elseif one == "" then
-        return lpegmatch(stripper,concat(lst,"/",2))
     else
-        return lpegmatch(deslasher,concat(lst,"/"))
+        return lpegmatch(deslasher,concat({  one, two, three, ... },"/"))
     end
 end
 
+-- or we can use this:
+--
+-- function file.join(...)
+--     local n = select("#",...)
+--     local one = select(1,...)
+--     if n == 1 then
+--         return one == "" and one or lpegmatch(stripper,one)
+--     end
+--     if one == "" then
+--         return lpegmatch(stripper,n > 2 and concat({ ... },"/",2) or select(2,...))
+--     end
+--     if lpegmatch(isnetwork,one) then
+--         local one = lpegmatch(reslasher,one)
+--         local two = lpegmatch(deslasher,n > 2 and concat({ ... },"/",2) or select(2,...))
+--         if lpegmatch(hasroot,two) then
+--             return one .. two
+--         else
+--             return one .. "/" .. two
+--         end
+--     elseif lpegmatch(isroot,one) then
+--         local two = lpegmatch(deslasher,n > 2 and concat({ ... },"/",2) or select(2,...))
+--         if lpegmatch(hasroot,two) then
+--             return two
+--         else
+--             return "/" .. two
+--         end
+--     else
+--         return lpegmatch(deslasher,concat({ ... },"/"))
+--     end
+-- end
+
+-- print(file.join("c:/whatever"))
 -- print(file.join("c:/whatever","name"))
 -- print(file.join("//","/y"))
 -- print(file.join("/","/y"))
@@ -412,6 +503,8 @@ end
 -- print(file.join("http://a","/y"))
 -- print(file.join("http:///a","/y"))
 -- print(file.join("//nas-1","/y"))
+-- print(file.join("//nas-1/a/b/c","/y"))
+-- print(file.join("\\\\nas-1\\a\\b\\c","\\y"))
 
 -- The previous one fails on "a.b/c"  so Taco came up with a split based
 -- variant. After some skyping we got it sort of compatible with the old
@@ -421,9 +514,14 @@ end
 -- finds were replaced by lpegs.
 
 local drivespec    = R("az","AZ")^1 * colon
-local anchors      = fwslash + drivespec
-local untouched    = periods + (1-period)^1 * P(-1)
-local splitstarter = (Cs(drivespec * (bwslash/"/" + fwslash)^0) + Cc(false)) * Ct(lpeg.splitat(S("/\\")^1))
+local anchors      = fwslash
+                   + drivespec
+local untouched    = periods
+                   + (1-period)^1 * P(-1)
+local mswindrive   = Cs(drivespec * (bwslash/"/" + fwslash)^0)
+local mswinuncpath = (bwslash + fwslash) * (bwslash + fwslash) * Cc("//")
+local splitstarter = (mswindrive + mswinuncpath + Cc(false))
+                   * Ct(lpeg.splitat(S("/\\")^1))
 local absolute     = fwslash
 
 function file.collapsepath(str,anchor) -- anchor: false|nil, true, "."
@@ -490,6 +588,7 @@ end
 -- test("a/./b/..") test("a/aa/../b/bb") test("a/.././././b/..") test("a/./././b/..")
 -- test("a/b/c/../..") test("./a/b/c/../..") test("a/b/c/../..")
 -- test("./a")
+-- test([[\\a.b.c\d\e]])
 
 local validchars = R("az","09","AZ","--","..")
 local pattern_a  = lpeg.replacer(1-validchars)
