@@ -70,6 +70,7 @@ function string.limit(str,n,sentinel) -- not utf proof
 end
 
 local stripper     = patterns.stripper
+local fullstripper = patterns.fullstripper
 local collapser    = patterns.collapser
 local longtostring = patterns.longtostring
 
@@ -77,6 +78,10 @@ function string.strip(str)
     return lpegmatch(stripper,str) or ""
 end
 
+function string.fullstrip(str)
+    return lpegmatch(fullstripper,str) or ""
+end
+
 function string.collapsespaces(str)
     return lpegmatch(collapser,str) or ""
 end
@@ -89,7 +94,9 @@ end
 --     return not find(str,"%S")
 -- end
 
-local pattern = P(" ")^0 * P(-1)
+local pattern = P(" ")^0 * P(-1) -- maybe also newlines
+
+-- patterns.onlyspaces = pattern
 
 function string.is_empty(str)
     if str == "" then
