@@ -280,10 +280,8 @@ end
 
 local pdfdocencodingvector, pdfdocencodingcapsule
 
--- The pdf doc encoding vector is needed in order to
--- trigger propper unicode. Interesting is that when
--- a glyph is not in the vector, it is still visible
--- as it is taken from some other font. Messy.
+-- The pdf doc encoding vector is needed in order to trigger propper unicode. Interesting is that when
+-- a glyph is not in the vector, it is still visible as it is taken from some other font. Messy.
 
 -- To be checked: only when text/line fields.
 
@@ -334,7 +332,7 @@ local function fieldsurrounding(specification)
     local fontsize        = specification.fontsize or "12pt"
     local fontstyle       = specification.fontstyle or "rm"
     local fontalternative = specification.fontalternative or "tf"
-    local colorvalue      = specification.colorvalue
+    local colorvalue      = tonumber(specification.colorvalue)
     local s = fontnames[fontstyle]
     if not s then
         fontstyle, s = "rm", fontnames.rm
