@@ -710,7 +710,9 @@ function codeinjections.setformat(s)
                 end
             end
             function codeinjections.setformat(noname)
-                report_backend("error, format is already set to %a, ignoring %a",formatname,noname.format)
+                if trace_format then
+                    report_backend("error, format is already set to %a, ignoring %a",formatname,noname.format)
+                end
             end
         else
             report_backend("error, format %a is not supported",format)
@@ -732,9 +734,11 @@ directives.register("backend.format", function(v) -- table !
     end
 end)
 
-function commands.setformat(s)
-    codeinjections.setformat(s)
-end
+interfaces.implement {
+    name      = "setformat",
+    actions   = codeinjections.setformat,
+    arguments = { { "*" } }
+}
 
 function codeinjections.getformatoption(key)
     return formatspecification and formatspecification[key]
