@@ -1,4 +1,4 @@
-if not modules then modules = { } end modules ['lxml-pth'] = {
+if not modules then modules = { } end modules ['lxml-lpt'] = {
     version   = 1.001,
     comment   = "this module is the basis for the lxml-* ones",
     author    = "Hans Hagen, PRAGMA-ADE, Hasselt NL",
@@ -10,11 +10,12 @@ if not modules then modules = { } end modules ['lxml-pth'] = {
 -- todo: B/C/[get first match]
 
 local concat, remove, insert = table.concat, table.remove, table.insert
-local type, next, tonumber, tostring, setmetatable, loadstring = type, next, tonumber, tostring, setmetatable, loadstring
+local type, next, tonumber, tostring, setmetatable, load, select = type, next, tonumber, tostring, setmetatable, load, select
 local format, upper, lower, gmatch, gsub, find, rep = string.format, string.upper, string.lower, string.gmatch, string.gsub, string.find, string.rep
 local lpegmatch, lpegpatterns = lpeg.match, lpeg.patterns
 
 local setmetatableindex = table.setmetatableindex
+local formatters = string.formatters -- no need (yet) as paths are cached anyway
 
 -- beware, this is not xpath ... e.g. position is different (currently) and
 -- we have reverse-sibling as reversed preceding sibling
@@ -82,7 +83,7 @@ local function fallback (t, name)
     if fn then
         t[name] = fn
     else
-        report_lpath("unknown sub finalizer '%s'",tostring(name))
+        report_lpath("unknown sub finalizer %a",name)
         fn = function() end
     end
     return fn
@@ -504,6 +505,9 @@ local function apply_expression(list,expression,order)
     return collected
 end
 
+-- this one can be made faster but there are not that many conversions so it doesn't
+-- really pay of
+
 local P, V, C, Cs, Cc, Ct, R, S, Cg, Cb = lpeg.P, lpeg.V, lpeg.C, lpeg.Cs, lpeg.Cc, lpeg.Ct, lpeg.R, lpeg.S, lpeg.Cg, lpeg.Cb
 
 local spaces     = S(" \n\r\t\f")^0
@@ -539,14 +543,22 @@ local lp_builtin = P (
 -- for the moment we keep namespaces with attributes
 
 local lp_attribute = (P("@") + P("attribute::")) / "" * Cc("(ll.at and ll.at['") * ((R("az","AZ") + S("-_:"))^1) * Cc("'])")
-local lp_fastpos_p = ((P("+")^0 * R("09")^1 * P(-1)) / function(s) return "l==" .. s end)
-local lp_fastpos_n = ((P("-")   * R("09")^1 * P(-1)) / function(s) return "(" .. s .. "<0 and (#list+".. s .. "==l))" end)
+
+----- lp_fastpos_p = (P("+")^0 * R("09")^1 * P(-1)) / function(s) return "l==" .. s end
+----- lp_fastpos_n = (P("-")   * R("09")^1 * P(-1)) / function(s) return "(" .. s .. "<0 and (#list+".. s .. "==l))" end
+
+local lp_fastpos_p = P("+")^0 * R("09")^1 * P(-1) / "l==%0"
+local lp_fastpos_n = P("-")   * R("09")^1 * P(-1) / "(%0<0 and (#list+%0==l))"
 local lp_fastpos   = lp_fastpos_n + lp_fastpos_p
+
 local lp_reserved  = C("and") + C("or") + C("not") + C("div") + C("mod") + C("true") + C("false")
 
-local lp_lua_function  = C(R("az","AZ","__")^1 * (P(".") * R("az","AZ","__")^1)^1) * ("(") / function(t) -- todo: better . handling
-    return t .. "("
-end
+-- local lp_lua_function = C(R("az","AZ","__")^1 * (P(".") * R("az","AZ","__")^1)^1) * ("(") / function(t) -- todo: better . handling
+--     return t .. "("
+-- end
+
+-- local lp_lua_function = (R("az","AZ","__")^1 * (P(".") * R("az","AZ","__")^1)^1) * ("(") / "%0("
+local lp_lua_function = Cs((R("az","AZ","__")^1 * (P(".") * R("az","AZ","__")^1)^1) * ("(")) / "%0"
 
 local lp_function  = C(R("az","AZ","__")^1) * P("(") / function(t) -- todo: better . handling
     if expressions[t] then
@@ -599,15 +611,12 @@ local converter = Cs (
 )
 
 cleaner = Cs ( (
---~     lp_fastpos +
+ -- lp_fastpos +
     lp_reserved +
     lp_number +
     lp_string +
 1 )^1 )
 
-
---~ expr
-
 local template_e = [[
     local expr = xml.expressions
     return function(list,ll,l,order)
@@ -662,6 +671,7 @@ local function errorrunner_e(str,cnv)
     end
     return false
 end
+
 local function errorrunner_f(str,arg)
     report_lpath("error in finalizer: %s(%s)",str,arg or "")
     return false
@@ -673,7 +683,7 @@ end
 
 local function register_expression(expression)
     local converted = lpegmatch(converter,expression)
-    local runner = loadstring(format(template_e,converted))
+    local runner = load(format(template_e,converted))
     runner = (runner and runner()) or function() errorrunner_e(expression,converted) end
     return { kind = "expression", expression = expression, converted = converted, evaluator = runner }
 end
@@ -681,9 +691,9 @@ end
 local function register_finalizer(protocol,name,arguments)
     local runner
     if arguments and arguments ~= "" then
-        runner = loadstring(format(template_f_y,protocol or xml.defaultprotocol,name,arguments))
+        runner = load(format(template_f_y,protocol or xml.defaultprotocol,name,arguments))
     else
-        runner = loadstring(format(template_f_n,protocol or xml.defaultprotocol,name))
+        runner = load(format(template_f_n,protocol or xml.defaultprotocol,name))
     end
     runner = (runner and runner()) or function() errorrunner_f(name,arguments) end
     return { kind = "finalizer", name = name, arguments = arguments, finalizer = runner }
@@ -798,7 +808,7 @@ local function nodesettostring(set,nodetest)
         if not ns or ns == "" then ns = "*" end
         if not tg or tg == "" then tg = "*" end
         tg = (tg == "@rt@" and "[root]") or format("%s:%s",ns,tg)
-        t[i] = (directive and tg) or format("not(%s)",tg)
+        t[#t+1] = (directive and tg) or format("not(%s)",tg)
     end
     if nodetest == false then
         return format("not(%s)",concat(t,"|"))
@@ -863,7 +873,7 @@ lpath = function (pattern) -- the gain of caching is rather minimal
                 local np = #parsed
                 if np == 0 then
                     parsed = { pattern = pattern, register_self, state = "parsing error" }
-                    report_lpath("parsing error in '%s'",pattern)
+                    report_lpath("parsing error in pattern: %s",pattern)
                     lshow(parsed)
                 else
                     -- we could have done this with a more complex parser but this
@@ -1031,37 +1041,6 @@ local function normal_apply(list,parsed,nofparsed,order)
     return collected
 end
 
---~ local function applylpath(list,pattern)
---~     -- we avoid an extra call
---~     local parsed = cache[pattern]
---~     if parsed then
---~         lpathcalls = lpathcalls + 1
---~         lpathcached = lpathcached + 1
---~     elseif type(pattern) == "table" then
---~         lpathcalls = lpathcalls + 1
---~         parsed = pattern
---~     else
---~         parsed = lpath(pattern) or pattern
---~     end
---~     if not parsed then
---~         return
---~     end
---~     local nofparsed = #parsed
---~     if nofparsed == 0 then
---~         return -- something is wrong
---~     end
---~     local one = list[1] -- we could have a third argument: isroot and list or list[1] or whatever we like ... todo
---~     if not one then
---~         return -- something is wrong
---~     elseif not trace_lpath then
---~         return normal_apply(list,parsed,nofparsed,one.mi)
---~     elseif trace_lprofile then
---~         return profiled_apply(list,parsed,nofparsed,one.mi)
---~     else
---~         return traced_apply(list,parsed,nofparsed,one.mi)
---~     end
---~ end
-
 local function applylpath(list,pattern)
     if not list then
         return
@@ -1107,6 +1086,7 @@ end
 expressions.child = function(e,pattern)
     return applylpath(e,pattern) -- todo: cache
 end
+
 expressions.count = function(e,pattern) -- what if pattern == empty or nil
     local collected = applylpath(e,pattern) -- todo: cache
     return pattern and (collected and #collected) or 0
@@ -1114,13 +1094,30 @@ end
 
 -- external
 
-expressions.oneof = function(s,...) -- slow
-    local t = {...} for i=1,#t do if s == t[i] then return true end end return false
+-- expressions.oneof = function(s,...)
+--     local t = {...}
+--     for i=1,#t do
+--         if s == t[i] then
+--             return true
+--         end
+--     end
+--     return false
+-- end
+
+expressions.oneof = function(s,...)
+    for i=1,select("#",...) do
+        if s == select(i,...) then
+            return true
+        end
+    end
+    return false
 end
+
 expressions.error = function(str)
     xml.errorhandler(format("unknown function in lpath expression: %s",tostring(str or "?")))
     return false
 end
+
 expressions.undefined = function(s)
     return s == nil
 end
@@ -1137,7 +1134,6 @@ expressions.print = function(...)
     return true
 end
 
-expressions.contains  = find
 expressions.find      = find
 expressions.upper     = upper
 expressions.lower     = lower
@@ -1161,10 +1157,14 @@ function expressions.contains(str,pattern)
     return false
 end
 
+function xml.expressions.idstring(str)
+    return type(str) == "string" and gsub(str,"^#","") or ""
+end
+
 -- user interface
 
 local function traverse(root,pattern,handle)
-    report_lpath("use 'xml.selection' instead for '%s'",pattern)
+ -- report_lpath("use 'xml.selection' instead for pattern: %s",pattern)
     local collected = applylpath(root,pattern)
     if collected then
         for c=1,#collected do
@@ -1197,15 +1197,15 @@ xml.selection     = selection          -- new method, simple handle
 
 -- generic function finalizer (independant namespace)
 
-local function dofunction(collected,fnc)
+local function dofunction(collected,fnc,...)
     if collected then
         local f = functions[fnc]
         if f then
             for c=1,#collected do
-                f(collected[c])
+                f(collected[c],...)
             end
         else
-            report_lpath("unknown function '%s'",fnc)
+            report_lpath("unknown function %a",fnc)
         end
     end
 end
@@ -1217,7 +1217,7 @@ finalizers.tex["function"] = dofunction
 
 expressions.text = function(e,n)
     local rdt = e.__p__.dt
-    return (rdt and rdt[n]) or ""
+    return rdt and rdt[n] or ""
 end
 
 expressions.name = function(e,n) -- ns + tg
@@ -1319,34 +1319,99 @@ end
 </typing>
 --ldx]]--
 
-local wrap, yield = coroutine.wrap, coroutine.yield
+-- local wrap, yield = coroutine.wrap, coroutine.yield
+-- local dummy = function() end
+--
+-- function xml.elements(root,pattern,reverse) -- r, d, k
+--     local collected = applylpath(root,pattern)
+--     if collected then
+--         if reverse then
+--             return wrap(function() for c=#collected,1,-1 do
+--                 local e = collected[c] local r = e.__p__ yield(r,r.dt,e.ni)
+--             end end)
+--         else
+--             return wrap(function() for c=1,#collected    do
+--                 local e = collected[c] local r = e.__p__ yield(r,r.dt,e.ni)
+--             end end)
+--         end
+--     end
+--     return wrap(dummy)
+-- end
+--
+-- function xml.collected(root,pattern,reverse) -- e
+--     local collected = applylpath(root,pattern)
+--     if collected then
+--         if reverse then
+--             return wrap(function() for c=#collected,1,-1 do yield(collected[c]) end end)
+--         else
+--             return wrap(function() for c=1,#collected    do yield(collected[c]) end end)
+--         end
+--     end
+--     return wrap(dummy)
+-- end
+
+-- faster:
+
+local dummy = function() end
 
 function xml.elements(root,pattern,reverse) -- r, d, k
     local collected = applylpath(root,pattern)
-    if collected then
-        if reverse then
-            return wrap(function() for c=#collected,1,-1 do
-                local e = collected[c] local r = e.__p__ yield(r,r.dt,e.ni)
-            end end)
-        else
-            return wrap(function() for c=1,#collected    do
-                local e = collected[c] local r = e.__p__ yield(r,r.dt,e.ni)
-            end end)
+    if not collected then
+        return dummy
+    end
+    local n = #collected
+    if n == 0 then
+        return dummy
+    end
+    if reverse then
+        local c = n + 1
+        return function()
+            if c > 1 then
+                c = c - 1
+                local e = collected[c]
+                local r = e.__p__
+                return r, r.dt, e.ni
+            end
+        end
+    else
+        local c = 0
+        return function()
+            if c < n then
+                c = c + 1
+                local e = collected[c]
+                local r = e.__p__
+                return r, r.dt, e.ni
+            end
         end
     end
-    return wrap(function() end)
 end
 
 function xml.collected(root,pattern,reverse) -- e
     local collected = applylpath(root,pattern)
-    if collected then
-        if reverse then
-            return wrap(function() for c=#collected,1,-1 do yield(collected[c]) end end)
-        else
-            return wrap(function() for c=1,#collected    do yield(collected[c]) end end)
+    if not collected then
+        return dummy
+    end
+    local n = #collected
+    if n == 0 then
+        return dummy
+    end
+    if reverse then
+        local c = n + 1
+        return function()
+            if c > 1 then
+                c = c - 1
+                return collected[c]
+            end
+        end
+    else
+        local c = 0
+        return function()
+            if c < n then
+                c = c + 1
+                return collected[c]
+            end
         end
     end
-    return wrap(function() end)
 end
 
 -- handy
@@ -1354,6 +1419,32 @@ end
 function xml.inspect(collection,pattern)
     pattern = pattern or "."
     for e in xml.collected(collection,pattern or ".") do
-        report_lpath("pattern %q\n\n%s\n",pattern,xml.tostring(e))
+        report_lpath("pattern: %s\n\n%s\n",pattern,xml.tostring(e))
     end
 end
+
+-- texy (see xfdf):
+
+local function split(e) -- todo: use helpers / lpeg
+    local dt = e.dt
+    if dt then
+        for i=1,#dt do
+            local dti = dt[i]
+            if type(dti) == "string" then
+                dti = gsub(dti,"^[\n\r]*(.-)[\n\r]*","%1")
+                dti = gsub(dti,"[\n\r]+","\n\n")
+                dt[i] = dti
+            else
+                split(dti)
+            end
+        end
+    end
+    return e
+end
+
+function xml.finalizers.paragraphs(c)
+    for i=1,#c do
+        split(c[i])
+    end
+    return c
+end
