@@ -18,30 +18,33 @@ if not modules then modules = { } end modules ['math-ini'] = {
 local formatters, find = string.formatters, string.find
 local utfchar, utfbyte, utflength = utf.char, utf.byte, utf.length
 local floor = math.floor
+local toboolean = toboolean
 
-local context           = context
-local commands          = commands
+local context               = context
+local commands              = commands
+local implement             = interfaces.implement
 
-local context_sprint    = context.sprint
------ context_fprint    = context.fprint -- a bit inefficient
+local context_sprint        = context.sprint
+----- context_fprint        = context.fprint -- a bit inefficient
+local ctx_doifelsesomething = commands.doifelsesomething
 
-local trace_defining    = false  trackers.register("math.defining", function(v) trace_defining = v end)
+local trace_defining        = false  trackers.register("math.defining", function(v) trace_defining = v end)
 
-local report_math       = logs.reporter("mathematics","initializing")
+local report_math           = logs.reporter("mathematics","initializing")
 
-mathematics             = mathematics or { }
-local mathematics       = mathematics
+mathematics                 = mathematics or { }
+local mathematics           = mathematics
 
-mathematics.extrabase   = 0xFE000 -- here we push some virtuals
-mathematics.privatebase = 0xFF000 -- here we push the ex
+mathematics.extrabase       = 0xFE000 -- here we push some virtuals
+mathematics.privatebase     = 0xFF000 -- here we push the ex
 
-local unsetvalue        = attributes.unsetvalue
-local allocate          = utilities.storage.allocate
-local chardata          = characters.data
+local unsetvalue            = attributes.unsetvalue
+local allocate              = utilities.storage.allocate
+local chardata              = characters.data
 
-local texsetattribute   = tex.setattribute
-local setmathcode       = tex.setmathcode
-local setdelcode        = tex.setdelcode
+local texsetattribute       = tex.setattribute
+local setmathcode           = tex.setmathcode
+local setdelcode            = tex.setdelcode
 
 local families = allocate {
     mr = 0,
@@ -464,7 +467,7 @@ end
 
 local function utfmathfiller(chr, default)
     local cd = somechar[chr]
-    local cmd = cd and (cd.mathfiller or cd.mathname)
+    local cmd = cd and cd.mathfiller -- or cd.mathname
     return cmd or default or ""
 end
 
@@ -476,20 +479,72 @@ mathematics.utfmathaccent  = utfmathaccent
 
 -- interfaced
 
-function commands.utfmathclass  (...) context(utfmathclass  (...)) end
-function commands.utfmathstretch(...) context(utfmathstretch(...)) end
-function commands.utfmathcommand(...) context(utfmathcommand(...)) end
-function commands.utfmathfiller (...) context(utfmathfiller (...)) end
+implement {
+    name      = "utfmathclass",
+    actions   = { utfmathclass, context },
+    arguments = "string"
+}
 
-function commands.doifelseutfmathaccent(chr,asked)
-    commands.doifelse(utfmathaccent(chr,nil,asked))
-end
+implement {
+    name      = "utfmathstretch",
+    actions   = { utfmathstretch, context },
+    arguments = "string"
+}
 
-function commands.utfmathcommandabove(asked) context(utfmathcommand(asked,nil,"topaccent","over" )) end
-function commands.utfmathcommandbelow(asked) context(utfmathcommand(asked,nil,"botaccent","under")) end
+implement {
+    name      = "utfmathcommand",
+    actions   = { utfmathcommand, context },
+    arguments = "string"
+}
 
-function commands.doifelseutfmathabove(chr) commands.doifelse(utfmathaccent(chr,nil,"topaccent","over" )) end
-function commands.doifelseutfmathbelow(chr) commands.doifelse(utfmathaccent(chr,nil,"botaccent","under")) end
+implement {
+    name      = "utfmathfiller",
+    actions   = { utfmathfiller, context },
+    arguments = "string"
+}
+
+implement {
+    name      = "utfmathcommandabove",
+    actions   = { utfmathcommand, context },
+    arguments = { "string", false, "'topaccent'","'over'" }
+}
+
+implement {
+    name      = "utfmathcommandbelow",
+    actions   = { utfmathcommand, context },
+    arguments = { "string", false, "'botaccent'","'under'" }
+}
+implement {
+    name      = "utfmathcommandfiller",
+    actions   = { utfmathfiller, context },
+    arguments = "string"
+}
+
+-- todo: make this a helper:
+
+implement {
+    name      = "doifelseutfmathabove",
+    actions   = { utfmathaccent, ctx_doifelsesomething },
+    arguments = { "string", false, "'topaccent'", "'over'" }
+}
+
+implement {
+    name      = "doifelseutfmathbelow",
+    actions   = { utfmathaccent, ctx_doifelsesomething },
+    arguments = { "string", false, "'botaccent'", "'under'" }
+}
+
+implement {
+    name      = "doifelseutfmathaccent",
+    actions   = { utfmathaccent, ctx_doifelsesomething },
+    arguments = "string",
+}
+
+implement {
+    name      = "doifelseutfmathfiller",
+    actions   = { utfmathfiller, ctx_doifelsesomething },
+    arguments = "string",
+}
 
 -- helpers
 --
@@ -599,9 +654,20 @@ local noffunctions = 1000 -- offset
 
 categories.functions = functions
 
-function commands.taggedmathfunction(tag,label,apply)
-    local delta = apply and 1000 or 0
-    if label then
+implement {
+    name      = "tagmfunctiontxt",
+    arguments = { "string", "conditional" },
+    actions   = function(tag,apply)
+        local delta = apply and 1000 or 0
+        texsetattribute(a_mathcategory,1000 + delta)
+    end
+}
+
+implement {
+    name      = "tagmfunctionlab",
+    arguments = { "string", "conditional" },
+    actions   = function(tag,apply)
+        local delta = apply and 1000 or 0
         local n = functions[tag]
         if not n then
             noffunctions = noffunctions + 1
@@ -611,18 +677,14 @@ function commands.taggedmathfunction(tag,label,apply)
         else
             texsetattribute(a_mathcategory,n + delta)
         end
-        context.mathlabeltext(tag)
-    else
-        texsetattribute(a_mathcategory,1000 + delta)
-        context(tag)
     end
-end
+}
 
 --
 
 local list
 
-function commands.resetmathattributes()
+function mathematics.resetattributes()
     if not list then
         list = { }
         for k, v in next, attributes.numbers do
@@ -635,3 +697,16 @@ function commands.resetmathattributes()
         texsetattribute(list[i],unsetvalue)
     end
 end
+
+implement {
+    name    = "resetmathattributes",
+    actions = mathematics.resetattributes
+}
+
+-- weird to do this here but it's a side affect of math anyway
+
+interfaces.implement {
+    name     = "enableasciimode",
+    onlyonce = true,
+    actions  = resolvers.macros.enablecomment,
+}
