@@ -329,7 +329,7 @@ handlers[50] = function() report_mptopdf("skipping special %s",50) end
 --end of not supported
 
 function mps.setrgbcolor(r,g,b) -- extra check
-    r, g = tonumber(r), tonumber(g) -- needed when we use lpeg
+    r, g, b = tonumber(r), tonumber(g), tonumber(b) -- needed when we use lpeg
     if r == 0.0123 and g < 0.1 then
         g, b = round(g*10000), round(b*10000)
         local s = specials[b]
@@ -411,7 +411,8 @@ function mps.fshow(str,font,scale) -- lpeg parser
     mps.textext(font,scale,lpegmatch(package,str))
 end
 
-local cnumber = lpegC(number)
+----- cnumber = lpegC(number)
+local cnumber = number/tonumber -- we now expect numbers (feeds into %F)
 local cstring = lpegC(nonspace)
 
 local specials           = (lpegP("%%MetaPostSpecials:") * sp * (cstring * sp^0)^0 * eol) / mps.specials
@@ -572,3 +573,11 @@ statistics.register("mps conversion time",function()
         return nil
     end
 end)
+
+-- interface
+
+interfaces.implement {
+    name      = "convertmpstopdf",
+    arguments = "string",
+    actions   = mptopdf.convertmpstopdf
+}
