@@ -232,7 +232,7 @@ return {
   "latelua",
   "luaescapestring",
   "luastartup",
-  "luatexdatestamp",
+  "luatexbanner",
   "luatexrevision",
   "luatexversion",
   "luafunction",
@@ -253,6 +253,8 @@ return {
   "suppressifcsnameerror",
   "suppresslongerror",
   "suppressoutererror",
+  "suppressmathparerror",
+  "matheqnogapstep",
   "synctex",
  },
  ["omega"]={
@@ -773,7 +775,7 @@ return {
   "lpcode",
   "luaescapestring",
   "luastartup",
-  "luatexdatestamp",
+  "luatexbanner",
   "luatexrevision",
   "luatexversion",
   "mag",
