@@ -12,18 +12,27 @@ if not modules then modules = { } end modules ['page-str'] = {
 
 local concat, insert, remove = table.concat, table.insert, table.remove
 
-local find_tail, write_node, free_node, copy_nodelist = node.slide, node.write, node.free, node.copy_list
-local vpack_nodelist, hpack_nodelist = node.vpack, node.hpack
-local texdimen, texbox = tex.dimen, tex.box
-local settings_to_array = utilities.parsers.settings_to_array
-
 local nodes, node = nodes, node
 
-local nodepool  = nodes.pool
-local tasks     = nodes.tasks
+local nodepool          = nodes.pool
+local tasks             = nodes.tasks
 
-local new_kern  = nodepool.kern
-local new_glyph = nodepool.glyph
+local implement         = interfaces.implement
+
+local new_kern          = nodepool.kern
+local new_glyph         = nodepool.glyph
+
+local slide_nodelist    = node.slide
+local write_node        = node.write
+local free_node         = node.free
+local copy_nodelist     = node.copy_list
+local vpack_nodelist    = node.vpack
+local hpack_nodelist    = node.hpack
+
+local settings_to_array = utilities.parsers.settings_to_array
+
+local texgetdimen       = tex.getdimen
+local texgetbox         = tex.getbox
 
 local trace_collecting = false  trackers.register("streams.collecting", function(v) trace_collecting = v end)
 local trace_flushing   = false  trackers.register("streams.flushing",   function(v) trace_flushing   = v end)
@@ -66,7 +75,7 @@ function streams.collect(head,where)
         end
         local last = dana[#dana]
         if last then
-            local tail = find_tail(last)
+            local tail = slide_nodelist(last)
             tail.next, head.prev = head, tail
         elseif last == false then
             dana[#dana] = head
@@ -175,7 +184,8 @@ function streams.synchronize(list) -- this is an experiment !
         if trace_flushing then
             report_streams("slot %s has max height %p and max depth %p",m,height,depth)
         end
-        local strutht, strutdp = texdimen.globalbodyfontstrutheight, texdimen.globalbodyfontstrutdepth
+        local strutht = texgetdimen("globalbodyfontstrutheight")
+        local strutdp = texgetdimen("globalbodyfontstrutdepth")
         local struthtdp = strutht + strutdp
         for i=1,#list do
             local name = list[i]
@@ -194,11 +204,11 @@ function streams.synchronize(list) -- this is an experiment !
                     else
                         -- this is not yet ok as we also need to keep an eye on vertical spacing
                         -- so we might need to do some splitting or whatever
-                        local tail = vbox.list and find_tail(vbox.list)
+                        local tail = vbox.list and slide_nodelist(vbox.list)
                         local n, delta = 0, delta_height -- for tracing
                         while delta > 0 do
                             -- we need to add some interline penalties
-                            local line = copy_nodelist(tex.box.strutbox)
+                            local line = copy_nodelist(texgetbox("strutbox"))
                             line.height, line.depth = strutht, strutdp
                             if tail then
                                 tail.next, line.prev = line, tail
@@ -227,6 +237,60 @@ tasks.disableaction("mvlbuilders", "streams.collect")
 
 function streams.initialize()
     tasks.enableaction ("mvlbuilders", "streams.collect")
+    function streams.initialize() end
 end
 
 -- todo: remove empty last { }'s
+-- todo: better names, enable etc
+
+implement {
+    name    = "initializestream",
+    actions = streams.initialize,
+    onlyonce = true,
+}
+
+implement {
+    name      = "enablestream",
+    actions   = streams.enable,
+    arguments = "string"
+}
+
+implement {
+    name      = "disablestream",
+    actions   = streams.disable
+}
+
+implement {
+    name      = "startstream",
+    actions   = streams.start,
+    arguments = "string"
+}
+
+implement {
+    name      = "stopstream",
+    actions   = streams.stop
+}
+
+implement {
+    name      = "flushstream",
+    actions   = streams.flush,
+    arguments = "string"
+}
+
+implement {
+    name      = "flushstreamcopy",
+    actions   = streams.flush,
+    arguments = { "string", true }
+}
+
+implement {
+    name      = "synchronizestream",
+    actions   = streams.synchronize,
+    arguments = "string"
+}
+
+implement {
+    name      = "pushstream",
+    actions   = streams.push,
+    arguments = "string"
+}
