@@ -14,13 +14,14 @@ local byte, lower = string.byte, string.lower
 -- to be redone: characters will become tagged spaces instead as then we keep track of
 -- spaceskip etc
 
--- todo: only setattr when export
+-- todo: only setattr when export / use properties
 
 local next = next
 
-trace_characters = false  trackers.register("typesetters.characters", function(v) trace_characters = v end)
+local trace_characters = false  trackers.register("typesetters.characters", function(v) trace_characters = v end)
+local trace_nbsp       = false  trackers.register("typesetters.nbsp",       function(v) trace_nbsp       = v end)
 
-report_characters = logs.reporter("typesetting","characters")
+local report_characters = logs.reporter("typesetting","characters")
 
 local nodes, node = nodes, node
 
@@ -39,6 +40,8 @@ local setattr            = nuts.setattr
 local getfont            = nuts.getfont
 local getchar            = nuts.getchar
 
+local setcolor           = nodes.tracers.colors.set
+
 local insert_node_before = nuts.insert_before
 local insert_node_after  = nuts.insert_after
 local remove_node        = nuts.remove
@@ -50,6 +53,8 @@ local tasks              = nodes.tasks
 local nodepool           = nuts.pool
 local new_penalty        = nodepool.penalty
 local new_glue           = nodepool.glue
+local new_kern           = nodepool.kern
+local new_rule           = nodepool.rule
 
 local nodecodes          = nodes.nodecodes
 local skipcodes          = nodes.skipcodes
@@ -107,13 +112,22 @@ local function inject_char_space(unicode,head,current,parent)
 end
 
 local function inject_nobreak_space(unicode,head,current,space,spacestretch,spaceshrink)
-    local attr = getfield(current,"attr")
-    local glue = new_glue(space,spacestretch,spaceshrink)
+    local attr    = getfield(current,"attr")
+    local glue    = new_glue(space,spacestretch,spaceshrink)
     local penalty = new_penalty(10000)
     setfield(glue,"attr",attr)
     setfield(current,"attr",nil)
-    setattr(glue,a_character,unicode)
+    setattr(glue,a_character,unicode) -- bombs
     head, current = insert_node_after(head,current,penalty)
+    if trace_nbsp then
+        local rule    = new_rule(space)
+        local kern    = new_kern(-space)
+        local penalty = new_penalty(10000)
+        setcolor(rule,"orange")
+        head, current = insert_node_after(head,current,rule)
+        head, current = insert_node_after(head,current,kern)
+        head, current = insert_node_after(head,current,penalty)
+    end
     head, current = insert_node_after(head,current,glue)
     return head, current
 end
