@@ -8,7 +8,6 @@ if not modules then modules = { } end modules ['strc-not'] = {
 
 local format = string.format
 local next = next
-local texcount = tex.count
 
 local trace_notes      = false  trackers.register("structures.notes",            function(v) trace_notes      = v end)
 local trace_references = false  trackers.register("structures.notes.references", function(v) trace_references = v end)
@@ -24,6 +23,9 @@ local notes            = structures.notes
 local references       = structures.references
 local counterspecials  = counters.specials
 
+local texgetcount      = tex.getcount
+local texgetbox        = tex.getbox
+
 notes.states           = notes.states or { }
 lists.enhancers        = lists.enhancers or { }
 
@@ -36,6 +38,8 @@ local variables  = interfaces.variables
 local context    = context
 local commands   = commands
 
+local implement  = interfaces.implement
+
 -- state: store, insert, postpone
 
 local function store(tag,n)
@@ -67,9 +71,11 @@ end
 
 notes.store = store
 
-function commands.storenote(tag,n)
-    context(store(tag,n))
-end
+implement {
+    name      = "storenote",
+    actions   = { store, context },
+    arguments = { "string", "integer" }
+}
 
 local function get(tag,n) -- tricky ... only works when defined
     local nd = notedata[tag]
@@ -91,10 +97,10 @@ end
 
 local function getn(tag)
     local nd = notedata[tag]
-    return (nd and #nd) or 0
+    return nd and #nd or 0
 end
 
-notes.get = get
+notes.get  = get
 notes.getn = getn
 
 -- we could make a special enhancer
@@ -106,9 +112,11 @@ end
 
 notes.listindex = listindex
 
-function commands.notelistindex(tag,n)
-    context(listindex(tag,n))
-end
+implement {
+    name      = "notelistindex",
+    actions   = { listindex, context },
+    arguments = { "string", "integer" }
+}
 
 local function setstate(tag,newkind)
     local state = notestates[tag]
@@ -142,18 +150,28 @@ end
 notes.setstate        = setstate
 notes.getstate        = getstate
 
-commands.setnotestate = setstate
+implement {
+    name      = "setnotestate",
+    actions   = setstate,
+    arguments = { "string", "string" }
+}
 
-function commands.getnotestate(tag)
-    context(getstate(tag))
-end
+implement {
+    name      = "getnotestate",
+    actions   = { getstate, context },
+    arguments = "string"
+}
 
 function notes.define(tag,kind,number)
     local state = setstate(tag,kind)
     state.number = number
 end
 
-commands.definenote = notes.define
+implement {
+    name      = "definenote",
+    actions   = notes.define,
+    arguments = { "string", "string", "integer" }
+}
 
 function notes.save(tag,newkind)
     local state = notestates[tag]
@@ -182,14 +200,14 @@ function notes.restore(tag,forcedstate)
     end
 end
 
-commands.savenote    = notes.save
-commands.restorenote = notes.restore
+implement { name = "savenote",    actions = notes.save,    arguments = { "string", "string" } }
+implement { name = "restorenote", actions = notes.restore, arguments = { "string", "string" } }
 
 local function hascontent(tag)
     local ok = notestates[tag]
     if ok then
         if ok.kind == "insert" then
-            ok = tex.box[ok.number]
+            ok = texgetbox(ok.number)
             if ok then
                 ok = tbs.list
                 ok = lst and lst.next
@@ -203,9 +221,11 @@ end
 
 notes.hascontent = hascontent
 
-function commands.doifnotecontent(tag)
-    commands.doif(hascontent(tag))
-end
+implement {
+    name      = "doifnotecontent",
+    actions   = { hascontent, commands.doif },
+    arguments = "string",
+}
 
 local function internal(tag,n)
     local nd = get(tag,n)
@@ -241,9 +261,11 @@ end
 
 notes.doifonsamepageasprevious = onsamepageasprevious
 
-function commands.doifnoteonsamepageasprevious(tag)
-    commands.doifelse(onsamepageasprevious(tag))
-end
+implement {
+    name      = "doifnoteonsamepageasprevious",
+    actions   = { onsamepageasprevious, commands.doifelse },
+    arguments = "string",
+}
 
 function notes.checkpagechange(tag) -- called before increment !
     local nd = notedata[tag] -- can be unset at first entry
@@ -257,7 +279,7 @@ function notes.checkpagechange(tag) -- called before increment !
             end
         elseif current then
             -- we need to locate the next one, best guess
-            if texcount.realpageno > current.pagenumber.number then
+            if texgetcount("realpageno") > current.pagenumber.number then
                 counters.reset(tag)
             end
         end
@@ -275,12 +297,15 @@ function notes.postpone()
     end
 end
 
-commands.postponenotes = notes.postpone
+implement {
+    name    = "postponenotes",
+    actions = notes.postpone
+}
 
 function notes.setsymbolpage(tag,n,l)
     local l = l or listindex(tag,n)
     if l then
-        local p = texcount.realpageno
+        local p = texgetcount("realpageno")
         if trace_notes or trace_references then
             report_notes("note %a of %a with list index %a gets symbol page %a",n,tag,l,p)
         end
@@ -295,7 +320,11 @@ function notes.setsymbolpage(tag,n,l)
     end
 end
 
-commands.setnotesymbolpage = notes.setsymbolpage
+implement {
+    name      = "setnotesymbolpage",
+    actions   = notes.setsymbolpage,
+    arguments = { "string", "integer" }
+}
 
 local function getsymbolpage(tag,n)
     local li = internal(tag,n)
@@ -349,11 +378,11 @@ notes.getsymbolpage = getsymbolpage
 notes.getnumberpage = getnumberpage
 notes.getdeltapage  = getdeltapage
 
-function commands.notesymbolpage(tag,n) context(getsymbolpage(tag,n)) end
-function commands.notenumberpage(tag,n) context(getnumberpage(tag,n)) end
-function commands.notedeltapage (tag,n) context(getdeltapage (tag,n)) end
+implement { name = "notesymbolpage", actions = { getsymbolpage, context }, arguments = { "string", "integer" } }
+implement { name = "notenumberpage", actions = { getnumberpage, context }, arguments = { "string", "integer" } }
+implement { name = "notedeltapage",  actions = { getdeltapage,  context }, arguments = { "string", "integer" } }
 
-function commands.flushnotes(tag,whatkind,how) -- store and postpone
+local function flushnotes(tag,whatkind,how) -- store and postpone
     local state = notestates[tag]
     local kind = state.kind
     if kind == whatkind then
@@ -382,7 +411,7 @@ function commands.flushnotes(tag,whatkind,how) -- store and postpone
                         local rp = get(tag,i)
                         rp = rp and rp.references
                         rp = rp and rp.symbolpage or 0
-                        if rp > texcount.realpageno then
+                        if rp > texgetcount("realpageno") then
                             state.start = i
                             return
                         end
@@ -409,15 +438,26 @@ function commands.flushnotes(tag,whatkind,how) -- store and postpone
     end
 end
 
-function commands.flushpostponednotes()
+local function flushpostponednotes()
     if trace_notes then
         report_notes("flushing all postponed notes")
     end
     for tag, _ in next, notestates do
-        commands.flushnotes(tag,"postpone")
+        flushnotes(tag,"postpone")
     end
 end
 
+implement {
+    name    = "flushpostponednotes",
+    actions = flushpostponednotes
+}
+
+implement {
+    name      = "flushnotes",
+    actions   = flushnotes,
+    arguments = { "string", "string",  "string" }
+}
+
 function notes.resetpostponed()
     if trace_notes then
         report_notes("resetting all postponed notes")
@@ -430,13 +470,17 @@ function notes.resetpostponed()
     end
 end
 
-function commands.notetitle(tag,n)
-    command.savedlisttitle(tag,notedata[tag][n])
-end
+implement {
+    name      = "notetitle",
+    actions   = function(tag,n) lists.savedlisttitle(tag,notedata[tag][n]) end,
+    arguments = { "string", "integer" }
+}
 
-function commands.noteprefixednumber(tag,n,spec)
-    commands.savedlistprefixednumber(tag,notedata[tag][n])
-end
+implement {
+    name      = "noteprefixednumber",
+    actions   = function(tag,n) lists.savedlistprefixednumber(tag,notedata[tag][n]) end,
+    arguments = { "string", "integer" }
+}
 
 function notes.internalid(tag,n)
     local nd = get(tag,n)
