@@ -6,75 +6,31 @@ if not modules then modules = { } end modules ['trac-log'] = {
     license   = "see context related readme files"
 }
 
--- if tex and (tex.jobname or tex.formatname) then
---
---     -- quick hack, awaiting speedup in engine (8 -> 6.4 sec for --make with console2)
---     -- still needed for luajittex
---
---     local texio_write_nl = texio.write_nl
---     local texio_write    = texio.write
---     local io_write       = io.write
-
---     local write_nl = function(target,...)
---         if not io_write then
---             io_write = io.write
---         end
---         if target == "term and log" then
---             texio_write_nl("log",...)
---             texio_write_nl("term","")
---             io_write(...)
---         elseif target == "log" then
---             texio_write_nl("log",...)
---         elseif target == "term" then
---             texio_write_nl("term","")
---             io_write(...)
---         else
---             texio_write_nl("log",target,...)
---             texio_write_nl("term","")
---             io_write(target,...)
---         end
---     end
-
---     local write = function(target,...)
---         if not io_write then
---             io_write = io.write
---         end
---         if target == "term and log" then
---             texio_write("log",...)
---             io_write(...)
---         elseif target == "log" then
---             texio_write("log",...)
---         elseif target == "term" then
---             io_write(...)
---         else
---             texio_write("log",target,...)
---             io_write(target,...)
---         end
---     end
-
---     texio.write    = write
---     texio.write_nl = write_nl
---
--- else
---
---     -- texlua or just lua
---
--- end
-
--- todo: less categories, more subcategories (e.g. nodes)
--- todo: split into basics and ctx specific
+-- In fact all writes could go through lua and we could write the console and
+-- terminal handler in lua then. Ok, maybe it's slower then, so a no-go.
 
+local next, type, select, print = next, type, select, print
 local write_nl, write = texio and texio.write_nl or print, texio and texio.write or io.write
 local format, gmatch, find = string.format, string.gmatch, string.find
 local concat, insert, remove = table.concat, table.insert, table.remove
 local topattern = string.topattern
-local texcount = tex and tex.count
-local next, type, select = next, type, select
 local utfchar = utf.char
+local datetime = os.date
+local openfile = io.open
 
 local setmetatableindex = table.setmetatableindex
 local formatters        = string.formatters
 
+local texgetcount       = tex and tex.getcount
+
+-- variant is set now
+
+local variant = "default"
+-- local variant = "ansi"
+
+-- todo: less categories, more subcategories (e.g. nodes)
+-- todo: split into basics and ctx specific
+
 --[[ldx--
 <p>This is a prelude to a more extensive logging module. We no longer
 provide <l n='xml'/> based logging as parsing is relatively easy anyway.</p>
@@ -97,29 +53,46 @@ wiki     : http://contextgarden.net
 -- function utilities.strings.chruni(s) return utfchar(s) .. " (U+" .. format("%05X",s) .. ")" end
 --
 -- utilities.strings.formatters.add (
---     string.formatters, "uni",
+--     string.formatters, "unichr",
 --     [[unichr(%s)]],
 --     [[local unichr = utilities.strings.unichr]]
 -- )
 --
 -- utilities.strings.formatters.add (
---     string.formatters, "chr",
+--     string.formatters, "chruni",
 --     [[chruni(%s)]],
 --     [[local chruni = utilities.strings.chruni]]
 -- )
 
-utilities.strings.formatters.add (
+formatters.add (
     formatters, "unichr",
     [["U+" .. format("%%05X",%s) .. " (" .. utfchar(%s) .. ")"]]
 )
 
-utilities.strings.formatters.add (
+formatters.add (
     formatters, "chruni",
     [[utfchar(%s) .. " (U+" .. format("%%05X",%s) .. ")"]]
 )
 
+-- function utilities.strings.unichk(s) return s <= 0xFFFF and ("U+" .. format("%05X",s) .. " (" .. utfchar(s) .. ")") or ("U+" .. format("%05X",s)) end
+-- function utilities.strings.chkuni(s) return s <= 0xFFFF and (utfchar(s) .. " (U+" .. format("%05X",s) .. ")") or ("U+" .. format("%05X",s)) end
+--
+-- utilities.strings.formatters.add (
+--     string.formatters, "unichk",
+--     [[unichk(%s)]],
+--     [[local unichk = utilities.strings.unichk]]
+-- )
+--
+-- utilities.strings.formatters.add (
+--     string.formatters, "chkuni",
+--     [[chkuni(%s)]],
+--     [[local chkuni = utilities.strings.chkuni]]
+-- )
+--
 -- print(formatters["Missing character %!chruni! in font."](234))
 -- print(formatters["Missing character %!unichr! in font."](234))
+-- print(formatters["Missing character %!chkuni! in font."](234))
+-- print(formatters["Missing character %!unichk! in font."](234))
 
 -- basic loggers
 
@@ -129,20 +102,127 @@ setmetatableindex(logs, function(t,k) t[k] = ignore ; return ignore end)
 
 local report, subreport, status, settarget, setformats, settranslations
 
-local direct, subdirect, writer, pushtarget, poptarget
+local direct, subdirect, writer, pushtarget, poptarget, setlogfile, settimedlog, setprocessor, setformatters, newline
+
+-- we use formatters but best check for % then because for simple messages but
+-- we don't want this overhead for single messages (not that there are that
+-- many; we could have a special weak table)
 
 if tex and (tex.jobname or tex.formatname) then
 
+    local function useluawrites()
+
+        -- quick hack, awaiting speedup in engine (8 -> 6.4 sec for --make with console2)
+        -- still needed for luajittex .. luatex should not have that ^^ mess
+
+        local texio_write_nl = texio.write_nl
+        local texio_write    = texio.write
+        local io_write       = io.write
+
+        write_nl = function(target,...)
+            if not io_write then
+                io_write = io.write
+            end
+            if target == "term and log" then
+                texio_write_nl("log",...)
+                texio_write_nl("term","")
+                io_write(...)
+            elseif target == "log" then
+                texio_write_nl("log",...)
+            elseif target == "term" then
+                texio_write_nl("term","")
+                io_write(...)
+            elseif target ~= "none" then
+                texio_write_nl("log",target,...)
+                texio_write_nl("term","")
+                io_write(target,...)
+            end
+        end
+
+        write = function(target,...)
+            if not io_write then
+                io_write = io.write
+            end
+            if target == "term and log" then
+                texio_write("log",...)
+                io_write(...)
+            elseif target == "log" then
+                texio_write("log",...)
+            elseif target == "term" then
+                io_write(...)
+            elseif target ~= "none" then
+                texio_write("log",target,...)
+                io_write(target,...)
+            end
+        end
+
+        texio.write    = write
+        texio.write_nl = write_nl
+
+        useluawrites   = ignore
+
+    end
+
  -- local format = string.formatter
 
-    local valueiskey   = { __index = function(t,k) t[k] = k return k end } -- will be helper
+    local whereto      = "both"
+    local target       = nil
+    local targets      = nil
 
-    local target       = "term and log"
+    local formats      = table.setmetatableindex("self")
+    local translations = table.setmetatableindex("self")
 
-    logs.flush         = io.flush
+    local report_yes, subreport_yes, direct_yes, subdirect_yes, status_yes
+    local report_nop, subreport_nop, direct_nop, subdirect_nop, status_nop
 
-    local formats      = { } setmetatable(formats,     valueiskey)
-    local translations = { } setmetatable(translations,valueiskey)
+    local variants = {
+        default = {
+            formats = {
+                report_yes    = formatters["%-15s > %s\n"],
+                report_nop    = formatters["%-15s >\n"],
+                direct_yes    = formatters["%-15s > %s"],
+                direct_nop    = formatters["%-15s >"],
+                subreport_yes = formatters["%-15s > %s > %s\n"],
+                subreport_nop = formatters["%-15s > %s >\n"],
+                subdirect_yes = formatters["%-15s > %s > %s"],
+                subdirect_nop = formatters["%-15s > %s >"],
+                status_yes    = formatters["%-15s : %s\n"],
+                status_nop    = formatters["%-15s :\n"],
+            },
+            targets = {
+                logfile  = "log",
+                log      = "log",
+                file     = "log",
+                console  = "term",
+                terminal = "term",
+                both     = "term and log",
+            },
+        },
+        ansi = {
+            formats = {
+                report_yes    = formatters["[0;33m%-15s [0;1m>[0m %s\n"],
+                report_nop    = formatters["[0;33m%-15s [0;1m>[0m\n"],
+                direct_yes    = formatters["[0;33m%-15s [0;1m>[0m %s"],
+                direct_nop    = formatters["[0;33m%-15s [0;1m>[0m"],
+                subreport_yes = formatters["[0;33m%-15s [0;1m>[0;35m %s [0;1m>[0m %s\n"],
+                subreport_nop = formatters["[0;33m%-15s [0;1m>[0;35m %s [0;1m>[0m\n"],
+                subdirect_yes = formatters["[0;33m%-15s [0;1m>[0;35m %s [0;1m>[0m %s"],
+                subdirect_nop = formatters["[0;33m%-15s [0;1m>[0;35m %s [0;1m>[0m"],
+                status_yes    = formatters["[0;33m%-15s [0;1m:[0m %s\n"],
+                status_nop    = formatters["[0;33m%-15s [0;1m:[0m\n"],
+            },
+            targets = {
+                logfile  = "none",
+                log      = "none",
+                file     = "none",
+                console  = "term",
+                terminal = "term",
+                both     = "term",
+            },
+        }
+    }
+
+    logs.flush = io.flush
 
     writer = function(...)
         write_nl(target,...)
@@ -152,96 +232,73 @@ if tex and (tex.jobname or tex.formatname) then
         write_nl(target,"\n")
     end
 
-    local f_one = formatters["%-15s > %s\n"]
-    local f_two = formatters["%-15s >\n"]
-
-    -- we can use formatters but best check for % then because for simple messages
-    -- we con't want this overhead for single messages (not that there are that
-    -- many; we could have a special weak table)
-
     report = function(a,b,c,...)
         if c then
-            write_nl(target,f_one(translations[a],formatters[formats[b]](c,...)))
+            write_nl(target,report_yes(translations[a],formatters[formats[b]](c,...)))
         elseif b then
-            write_nl(target,f_one(translations[a],formats[b]))
+            write_nl(target,report_yes(translations[a],formats[b]))
         elseif a then
-            write_nl(target,f_two(translations[a]))
+            write_nl(target,report_nop(translations[a]))
         else
             write_nl(target,"\n")
         end
     end
 
-    local f_one = formatters["%-15s > %s"]
-    local f_two = formatters["%-15s >"]
-
     direct = function(a,b,c,...)
         if c then
-            return f_one(translations[a],formatters[formats[b]](c,...))
+            return direct_yes(translations[a],formatters[formats[b]](c,...))
         elseif b then
-            return f_one(translations[a],formats[b])
+            return direct_yes(translations[a],formats[b])
         elseif a then
-            return f_two(translations[a])
+            return direct_nop(translations[a])
         else
             return ""
         end
     end
 
-    local f_one = formatters["%-15s > %s > %s\n"]
-    local f_two = formatters["%-15s > %s >\n"]
-
     subreport = function(a,s,b,c,...)
         if c then
-            write_nl(target,f_one(translations[a],translations[s],formatters[formats[b]](c,...)))
+            write_nl(target,subreport_yes(translations[a],translations[s],formatters[formats[b]](c,...)))
         elseif b then
-            write_nl(target,f_one(translations[a],translations[s],formats[b]))
+            write_nl(target,subreport_yes(translations[a],translations[s],formats[b]))
         elseif a then
-            write_nl(target,f_two(translations[a],translations[s]))
+            write_nl(target,subreport_nop(translations[a],translations[s]))
         else
             write_nl(target,"\n")
         end
     end
 
-    local f_one = formatters["%-15s > %s > %s"]
-    local f_two = formatters["%-15s > %s >"]
-
     subdirect = function(a,s,b,c,...)
         if c then
-            return f_one(translations[a],translations[s],formatters[formats[b]](c,...))
+            return subdirect_yes(translations[a],translations[s],formatters[formats[b]](c,...))
         elseif b then
-            return f_one(translations[a],translations[s],formats[b])
+            return subdirect_yes(translations[a],translations[s],formats[b])
         elseif a then
-            return f_two(translations[a],translations[s])
+            return subdirect_nop(translations[a],translations[s])
         else
             return ""
         end
     end
 
-    local f_one = formatters["%-15s : %s\n"]
-    local f_two = formatters["%-15s :\n"]
-
     status = function(a,b,c,...)
         if c then
-            write_nl(target,f_one(translations[a],formatters[formats[b]](c,...)))
+            write_nl(target,status_yes(translations[a],formatters[formats[b]](c,...)))
         elseif b then
-            write_nl(target,f_one(translations[a],formats[b]))
+            write_nl(target,status_yes(translations[a],formats[b]))
         elseif a then
-            write_nl(target,f_two(translations[a]))
+            write_nl(target,status_nop(translations[a]))
         else
             write_nl(target,"\n")
         end
     end
 
-    local targets = {
-        logfile  = "log",
-        log      = "log",
-        file     = "log",
-        console  = "term",
-        terminal = "term",
-        both     = "term and log",
-    }
-
-    settarget = function(whereto)
-        target = targets[whereto or "both"] or targets.both
+    settarget = function(askedwhereto)
+        whereto = askedwhereto or whereto or "both"
+        target = targets[whereto]
+        if not target then
+            whereto   = "both"
+            target = targets[whereto]
+        end
         if target == "term" or target == "term and log" then
             logs.flush = io.flush
         else
@@ -270,56 +327,129 @@ if tex and (tex.jobname or tex.formatname) then
         translations = t
     end
 
+    setprocessor = function(f)
+        local writeline = write_nl
+        write_nl = function(target,...)
+            writeline(target,f(...))
+        end
+    end
+
+    setformatters = function(specification)
+        local t = nil
+        local f = nil
+        local d = variants.default
+        if not specification then
+            --
+        elseif type(specification) == "table" then
+            t = specification.targets
+            f = specification.formats or specification
+        else
+            local v = variants[specification]
+            if v then
+                t = v.targets
+                f = v.formats
+                variant = specification
+            end
+        end
+        targets = t or d.targets
+        target = targets[whereto] or target
+        if f then
+            d = d.formats
+        else
+            f = d.formats
+            d = f
+        end
+        setmetatableindex(f,d)
+        report_yes    = f.report_yes
+        report_nop    = f.report_nop
+        subreport_yes = f.subreport_yes
+        subreport_nop = f.subreport_nop
+        direct_yes    = f.direct_yes
+        direct_nop    = f.direct_nop
+        subdirect_yes = f.subdirect_yes
+        subdirect_nop = f.subdirect_nop
+        status_yes    = f.status_yes
+        status_nop    = f.status_nop
+        if variant == "ansi" then
+            useluawrites() -- because tex escapes ^^
+        end
+        settarget(whereto)
+    end
+
+    setformatters(variant)
+
+    setlogfile  = ignore
+    settimedlog = ignore
+
 else
 
+    local report_yes, subreport_yes, status_yes
+    local report_nop, subreport_nop, status_nop
+
+    local variants = {
+        default = {
+            formats = {
+                report_yes    = formatters["%-15s | %s"],
+                report_nop    = formatters["%-15s |"],
+                subreport_yes = formatters["%-15s | %s | %s"],
+                subreport_nop = formatters["%-15s | %s |"],
+                status_yes    = formatters["%-15s : %s\n"],
+                status_nop    = formatters["%-15s :\n"],
+            },
+        },
+        ansi = {
+            formats = {
+                report_yes    = formatters["[0;32m%-15s [0;1m|[0m %s"],
+                report_nop    = formatters["[0;32m%-15s [0;1m|[0m"],
+                subreport_yes = formatters["[0;32m%-15s [0;1m|[0;31m %s [0;1m|[0m %s"],
+                subreport_nop = formatters["[0;32m%-15s [0;1m|[0;31m %s [0;1m|[0m"],
+                status_yes    = formatters["[0;32m%-15s [0;1m:[0m %s\n"],
+                status_nop    = formatters["[0;32m%-15s [0;1m:[0m\n"],
+            },
+        },
+    }
+
     logs.flush = ignore
 
-    writer = write_nl
+    writer = function(s)
+        write_nl(s)
+    end
 
     newline = function()
         write_nl("\n")
     end
 
-    local f_one = formatters["%-15s | %s"]
-    local f_two = formatters["%-15s |"]
-
     report = function(a,b,c,...)
         if c then
-            write_nl(f_one(a,formatters[b](c,...)))
+            write_nl(report_yes(a,formatters[b](c,...)))
         elseif b then
-            write_nl(f_one(a,b))
+            write_nl(report_yes(a,b))
         elseif a then
-            write_nl(f_two(a))
+            write_nl(report_nop(a))
         else
             write_nl("")
         end
     end
 
-    local f_one = formatters["%-15s | %s | %s"]
-    local f_two = formatters["%-15s | %s |"]
-
     subreport = function(a,sub,b,c,...)
         if c then
-            write_nl(f_one(a,sub,formatters[b](c,...)))
+            write_nl(subreport_yes(a,sub,formatters[b](c,...)))
         elseif b then
-            write_nl(f_one(a,sub,b))
+            write_nl(subreport_yes(a,sub,b))
         elseif a then
-            write_nl(f_two(a,sub))
+            write_nl(subreport_nop(a,sub))
         else
             write_nl("")
         end
     end
 
-    local f_one = formatters["%-15s : %s\n"]
-    local f_two = formatters["%-15s :\n"]
-
     status = function(a,b,c,...) -- not to be used in lua anyway
         if c then
-            write_nl(f_one(a,formatters[b](c,...)))
+            write_nl(status_yes(a,formatters[b](c,...)))
         elseif b then
-            write_nl(f_one(a,b)) -- b can have %'s
+            write_nl(status_yes(a,b)) -- b can have %'s
         elseif a then
-            write_nl(f_two(a))
+            write_nl(status_nop(a))
         else
             write_nl("\n")
         end
@@ -334,6 +464,74 @@ else
     setformats      = ignore
     settranslations = ignore
 
+    setprocessor = function(f)
+        local writeline = write_nl
+        write_nl = function(s)
+            writeline(f(s))
+        end
+    end
+
+    setformatters = function(specification)
+        local f = nil
+        local d = variants.default
+        if specification then
+            if type(specification) == "table" then
+                f = specification.formats or specification
+            else
+                local v = variants[specification]
+                if v then
+                    f = v.formats
+                end
+            end
+        end
+        if f then
+            d = d.formats
+        else
+            f = d.formats
+            d = f
+        end
+        setmetatableindex(f,d)
+        report_yes    = f.report_yes
+        report_nop    = f.report_nop
+        subreport_yes = f.subreport_yes
+        subreport_nop = f.subreport_nop
+        status_yes    = f.status_yes
+        status_nop    = f.status_nop
+    end
+
+    setformatters(variant)
+
+    setlogfile = function(name,keepopen)
+        if name and name ~= "" then
+            local localtime = os.localtime
+            local writeline = write_nl
+            if keepopen then
+                local f = io.open(name,"ab")
+                write_nl = function(s)
+                    writeline(s)
+                    f:write(localtime()," | ",s,"\n")
+                end
+            else
+                write_nl = function(s)
+                    writeline(s)
+                    local f = io.open(name,"ab")
+                    f:write(localtime()," | ",s,"\n")
+                    f:close()
+                end
+            end
+        end
+        setlogfile = ignore
+    end
+
+    settimedlog = function()
+        local localtime = os.localtime
+        local writeline = write_nl
+        write_nl = function(s)
+            writeline(localtime() .. " | " .. s)
+        end
+        settimedlog = ignore
+    end
+
 end
 
 logs.report          = report
@@ -345,6 +543,11 @@ logs.poptarget       = poptarget
 logs.setformats      = setformats
 logs.settranslations = settranslations
 
+logs.setlogfile      = setlogfile
+logs.settimedlog     = settimedlog
+logs.setprocessor    = setprocessor
+logs.setformatters   = setformatters
+
 logs.direct          = direct
 logs.subdirect       = subdirect
 logs.writer          = writer
@@ -440,9 +643,10 @@ local function setblocked(category,value)
             v.state = value
         end
     else
-        states = utilities.parsers.settings_to_hash(category)
+        states = utilities.parsers.settings_to_hash(category,type(states)=="table" and states or nil)
         for c, _ in next, states do
-            if data[c] then
+            local v = data[c]
+            if v then
                 v.state = value
             else
                 c = topattern(c,true,true)
@@ -529,8 +733,9 @@ local report_pages = logs.reporter("pages") -- not needed but saves checking whe
 local real, user, sub
 
 function logs.start_page_number()
-    real, user, sub = texcount.realpageno, texcount.userpageno, texcount.subpageno
---     real, user, sub = 0, 0, 0
+    real = texgetcount("realpageno")
+    user = texgetcount("userpageno")
+    sub  = texgetcount("subpageno")
 end
 
 local timing    = false
@@ -626,13 +831,13 @@ logs.simpleline = simple
 
 -- obsolete
 
-function logs.setprogram  () end -- obsolete
-function logs.extendbanner() end -- obsolete
-function logs.reportlines () end -- obsolete
-function logs.reportbanner() end -- obsolete
-function logs.reportline  () end -- obsolete
-function logs.simplelines () end -- obsolete
-function logs.help        () end -- obsolete
+logs.setprogram   = ignore -- obsolete
+logs.extendbanner = ignore -- obsolete
+logs.reportlines  = ignore -- obsolete
+logs.reportbanner = ignore -- obsolete
+logs.reportline   = ignore -- obsolete
+logs.simplelines  = ignore -- obsolete
+logs.help         = ignore -- obsolete
 
 -- applications
 
@@ -745,10 +950,12 @@ end
 --     logs.system(syslogname,"context","test","fonts","font %s recached due to newer version (%s)","blabla","123")
 -- end
 
-function logs.system(whereto,process,jobname,category,...)
-    local message = formatters["%s %s => %s => %s => %s\r"](os.date("%d/%m/%y %H:%m:%S"),process,jobname,category,format(...))
+local f_syslog = formatters["%s %s => %s => %s => %s\r"]
+
+function logs.system(whereto,process,jobname,category,fmt,arg,...)
+    local message = f_syslog(datetime("%d/%m/%y %H:%m:%S"),process,jobname,category,arg == nil and fmt or format(fmt,arg,...))
     for i=1,10 do
-        local f = io.open(whereto,"a") -- we can consider keeping the file open
+        local f = openfile(whereto,"a") -- we can consider keeping the file open
         if f then
             f:write(message)
             f:close()
