@@ -9,8 +9,9 @@ if not modules then modules = { } end modules ['util-prs'] = {
 local lpeg, table, string = lpeg, table, string
 local P, R, V, S, C, Ct, Cs, Carg, Cc, Cg, Cf, Cp = lpeg.P, lpeg.R, lpeg.V, lpeg.S, lpeg.C, lpeg.Ct, lpeg.Cs, lpeg.Carg, lpeg.Cc, lpeg.Cg, lpeg.Cf, lpeg.Cp
 local lpegmatch, lpegpatterns = lpeg.match, lpeg.patterns
-local concat, format, gmatch, find = table.concat, string.format, string.gmatch, string.find
+local concat, gmatch, find = table.concat, string.gmatch, string.find
 local tostring, type, next, rawset = tostring, type, next, rawset
+local mod, div = math.mod, math.div
 
 utilities         = utilities or {}
 local parsers     = utilities.parsers or { }
@@ -20,6 +21,8 @@ parsers.patterns  = patterns
 
 local setmetatableindex = table.setmetatableindex
 local sortedhash        = table.sortedhash
+local sortedkeys        = table.sortedkeys
+local tohash            = table.tohash
 
 -- we share some patterns
 
@@ -39,8 +42,8 @@ local newline     = lpegpatterns.newline
 local anything    = lpegpatterns.anything
 local endofstring = lpegpatterns.endofstring
 
-local nobrace     = 1 - ( lbrace  + rbrace )
-local noparent    = 1 - ( lparent + rparent)
+local nobrace     = 1 - (lbrace  + rbrace )
+local noparent    = 1 - (lparent + rparent)
 
 -- we could use a Cf Cg construct
 
@@ -93,9 +96,7 @@ patterns.settings_to_hash_b = pattern_b_s
 patterns.settings_to_hash_c = pattern_c_s
 
 function parsers.make_settings_to_hash_pattern(set,how)
-    if type(str) == "table" then
-        return set
-    elseif how == "strict" then
+    if how == "strict" then
         return (pattern_c/set)^1
     elseif how == "tolerant" then
         return (pattern_b/set)^1
@@ -105,7 +106,9 @@ function parsers.make_settings_to_hash_pattern(set,how)
 end
 
 function parsers.settings_to_hash(str,existing)
-    if type(str) == "table" then
+    if not str or str == "" then
+        return { }
+    elseif type(str) == "table" then
         if existing then
             for k, v in next, str do
                 existing[k] = v
@@ -114,17 +117,17 @@ function parsers.settings_to_hash(str,existing)
         else
             return str
         end
-    elseif str and str ~= "" then
+    else
         hash = existing or { }
         lpegmatch(pattern_a_s,str)
         return hash
-    else
-        return { }
     end
 end
 
 function parsers.settings_to_hash_tolerant(str,existing)
-    if type(str) == "table" then
+    if not str or str == "" then
+        return { }
+    elseif type(str) == "table" then
         if existing then
             for k, v in next, str do
                 existing[k] = v
@@ -133,17 +136,17 @@ function parsers.settings_to_hash_tolerant(str,existing)
         else
             return str
         end
-    elseif str and str ~= "" then
+    else
         hash = existing or { }
         lpegmatch(pattern_b_s,str)
         return hash
-    else
-        return { }
     end
 end
 
 function parsers.settings_to_hash_strict(str,existing)
-    if type(str) == "table" then
+    if not str or str == "" then
+        return nil
+    elseif type(str) == "table" then
         if existing then
             for k, v in next, str do
                 existing[k] = v
@@ -156,8 +159,6 @@ function parsers.settings_to_hash_strict(str,existing)
         hash = existing or { }
         lpegmatch(pattern_c_s,str)
         return next(hash) and hash
-    else
-        return nil
     end
 end
 
@@ -166,28 +167,88 @@ local value     = P(lbrace * C((nobrace + nestedbraces)^0) * rbrace)
                 + C((nestedbraces + (1-comma))^0)
 local pattern   = spaces * Ct(value*(separator*value)^0)
 
--- "aap, {noot}, mies" : outer {} removes, leading spaces ignored
+-- "aap, {noot}, mies" : outer {} removed, leading spaces ignored
 
 patterns.settings_to_array = pattern
 
 -- we could use a weak table as cache
 
 function parsers.settings_to_array(str,strict)
-    if type(str) == "table" then
-        return str
-    elseif not str or str == "" then
+    if not str or str == "" then
         return { }
+    elseif type(str) == "table" then
+        return str
     elseif strict then
-        if find(str,"{") then
+        if find(str,"{",1,true) then
             return lpegmatch(pattern,str)
         else
             return { str }
         end
-    else
+    elseif find(str,",",1,true) then
         return lpegmatch(pattern,str)
+    else
+        return { str }
     end
 end
 
+-- this one also strips end spaces before separators
+--
+-- "{123} , 456  " -> "123" "456"
+
+-- local separator = space^0 * comma * space^0
+-- local value     = P(lbrace * C((nobrace + nestedbraces)^0) * rbrace)
+--                 + C((nestedbraces + (1-(space^0*(comma+P(-1)))))^0)
+-- local withvalue = Carg(1) * value / function(f,s) return f(s) end
+-- local pattern_a = spaces * Ct(value*(separator*value)^0)
+-- local pattern_b = spaces * withvalue * (separator*withvalue)^0
+
+local cache_a = { }
+local cache_b = { }
+
+function parsers.groupedsplitat(symbol,withaction)
+    if not symbol then
+        symbol = ","
+    end
+    local pattern = (withaction and cache_b or cache_a)[symbol]
+    if not pattern then
+        local symbols   = S(symbol)
+        local separator = space^0 * symbols * space^0
+        local value     = P(lbrace * C((nobrace + nestedbraces)^0) * rbrace)
+                        + C((nestedbraces + (1-(space^0*(symbols+P(-1)))))^0)
+        if withaction then
+            local withvalue = Carg(1) * value / function(f,s) return f(s) end
+            pattern = spaces * withvalue * (separator*withvalue)^0
+            cache_b[symbol] = pattern
+        else
+            pattern = spaces * Ct(value*(separator*value)^0)
+            cache_a[symbol] = pattern
+        end
+    end
+    return pattern
+end
+
+local pattern_a = parsers.groupedsplitat(",",false)
+local pattern_b = parsers.groupedsplitat(",",true)
+
+function parsers.stripped_settings_to_array(str)
+    if not str or str == "" then
+        return { }
+    else
+        return lpegmatch(pattern_a,str)
+    end
+end
+
+function parsers.process_stripped_settings(str,action)
+    if not str or str == "" then
+        return { }
+    else
+        return lpegmatch(pattern_b,str,1,action)
+    end
+end
+
+-- parsers.process_stripped_settings("{123} , 456  ",function(s) print("["..s.."]") end)
+-- parsers.process_stripped_settings("123 , 456  ",function(s) print("["..s.."]") end)
+
 local function set(t,v)
     t[#t+1] = v
 end
@@ -201,8 +262,8 @@ end
 
 function parsers.hash_to_string(h,separator,yes,no,strict,omit)
     if h then
-        local t, tn, s = { }, 0, table.sortedkeys(h)
-        omit = omit and table.tohash(omit)
+        local t, tn, s = { }, 0, sortedkeys(h)
+        omit = omit and tohash(omit)
         for i=1,#s do
             local key = s[i]
             if not omit or not omit[key] then
@@ -240,15 +301,25 @@ function parsers.array_to_string(a,separator)
     end
 end
 
-function parsers.settings_to_set(str,t) -- tohash? -- todo: lpeg -- duplicate anyway
-    t = t or { }
---  for s in gmatch(str,"%s*([^, ]+)") do -- space added
-    for s in gmatch(str,"[^, ]+") do -- space added
-        t[s] = true
-    end
-    return t
+-- function parsers.settings_to_set(str,t) -- tohash? -- todo: lpeg -- duplicate anyway
+--     if str then
+--         t = t or { }
+--         for s in gmatch(str,"[^, ]+") do -- space added
+--             t[s] = true
+--         end
+--         return t
+--     else
+--         return { }
+--     end
+-- end
+
+local pattern = Cf(Ct("") * Cg(C((1-S(", "))^1) * S(", ")^0 * Cc(true))^1,rawset)
+
+function utilities.parsers.settings_to_set(str,t)
+    return str and lpegmatch(pattern,str) or { }
 end
 
+
 function parsers.simple_hash_to_string(h, separator)
     local t, tn = { }, 0
     for k, v in sortedhash(h) do
@@ -260,6 +331,22 @@ function parsers.simple_hash_to_string(h, separator)
     return concat(t,separator or ",")
 end
 
+-- for mtx-context etc: aaaa bbbb cccc=dddd eeee=ffff
+
+local str      = Cs(lpegpatterns.unquoted) + C((1-whitespace-equal)^1)
+local setting  = Cf( Carg(1) * (whitespace^0 * Cg(str * whitespace^0 * (equal * whitespace^0 * str + Cc(""))))^1,rawset)
+local splitter = setting^1
+
+function utilities.parsers.options_to_hash(str,target)
+    return str and lpegmatch(splitter,str,1,target or { }) or { }
+end
+
+local splitter = lpeg.tsplitat(" ")
+
+function utilities.parsers.options_to_array(str)
+    return str and lpegmatch(splitter,str) or { }
+end
+
 -- for chem (currently one level)
 
 local value     = P(lbrace * C((nobrace + nestedbraces)^0) * rbrace)
@@ -391,7 +478,7 @@ local defaultspecification = { separator = ",", quote = '"' }
 -- database module
 
 function parsers.csvsplitter(specification)
-    specification   = specification and table.setmetatableindex(specification,defaultspecification) or defaultspecification
+    specification   = specification and setmetatableindex(specification,defaultspecification) or defaultspecification
     local separator = specification.separator
     local quotechar = specification.quote
     local separator = S(separator ~= "" and separator or ",")
@@ -409,7 +496,7 @@ function parsers.csvsplitter(specification)
         end
         whatever = quotedata + whatever
     end
-    local parser = Ct((Ct(whatever * (separator * whatever)^0) * S("\n\r"))^0 )
+    local parser = Ct((Ct(whatever * (separator * whatever)^0) * S("\n\r")^1)^0 )
     return function(data)
         return lpegmatch(parser,data)
     end
@@ -430,7 +517,7 @@ end
 -- local list, names = mycsvsplitter(crap)        inspect(list) inspect(names)
 
 function parsers.rfc4180splitter(specification)
-    specification     = specification and table.setmetatableindex(specification,defaultspecification) or defaultspecification
+    specification     = specification and setmetatableindex(specification,defaultspecification) or defaultspecification
     local separator   = specification.separator --> rfc: COMMA
     local quotechar   = P(specification.quote)  -->      DQUOTE
     local dquotechar  = quotechar * quotechar   -->      2DQUOTE
@@ -440,10 +527,10 @@ function parsers.rfc4180splitter(specification)
                       * Cs((dquotechar + (1 - quotechar))^0)
                       * quotechar
     local non_escaped = C((1 - quotechar - newline - separator)^1)
-    local field       = escaped + non_escaped
-    local record      = Ct((field * separator^-1)^1)
+    local field       = escaped + non_escaped + Cc("")
+    local record      = Ct(field * (separator * field)^1)
     local headerline  = record * Cp()
-    local wholeblob   = Ct((newline^-1 * record)^0)
+    local wholeblob   = Ct((newline^(specification.strict and -1 or 1) * record)^0)
     return function(data,getheader)
         if getheader then
             local header, position = lpegmatch(headerline,data)
@@ -497,8 +584,8 @@ end
 
 --
 
-local pattern_math = Cs((P("%")/"\\percent " +  P("^")           * Cc("{") * lpegpatterns.integer * Cc("}") + P(1))^0)
-local pattern_text = Cs((P("%")/"\\percent " + (P("^")/"\\high") * Cc("{") * lpegpatterns.integer * Cc("}") + P(1))^0)
+local pattern_math = Cs((P("%")/"\\percent " +  P("^")           * Cc("{") * lpegpatterns.integer * Cc("}") + anything)^0)
+local pattern_text = Cs((P("%")/"\\percent " + (P("^")/"\\high") * Cc("{") * lpegpatterns.integer * Cc("}") + anything)^0)
 
 patterns.unittotex = pattern
 
@@ -506,7 +593,7 @@ function parsers.unittotex(str,textmode)
     return lpegmatch(textmode and pattern_text or pattern_math,str)
 end
 
-local pattern = Cs((P("^") / "<sup>" * lpegpatterns.integer * Cc("</sup>") + P(1))^0)
+local pattern = Cs((P("^") / "<sup>" * lpegpatterns.integer * Cc("</sup>") + anything)^0)
 
 function parsers.unittoxml(str)
     return lpegmatch(pattern,str)
@@ -515,10 +602,10 @@ end
 -- print(utilities.parsers.unittotex("10^-32 %"),utilities.parsers.unittoxml("10^32 %"))
 
 local cache   = { }
-local spaces  = lpeg.patterns.space^0
+local spaces  = lpegpatterns.space^0
 local dummy   = function() end
 
-table.setmetatableindex(cache,function(t,k)
+setmetatableindex(cache,function(t,k)
     local separator = P(k)
     local value     = (1-separator)^0
     local pattern   = spaces * C(value) * separator^0 * Cp()
@@ -568,7 +655,7 @@ local function fetch(t,name)
     return t[name] or { }
 end
 
-function process(result,more)
+local function process(result,more)
     for k, v in next, more do
         result[k] = v
     end
@@ -590,3 +677,40 @@ end
 -- }
 --
 -- inspect(utilities.parsers.mergehashes(t,"aa, bb, cc"))
+
+function utilities.parsers.runtime(time)
+    if not time then
+        time = os.runtime()
+    end
+    local days = div(time,24*60*60)
+    time = mod(time,24*60*60)
+    local hours = div(time,60*60)
+    time = mod(time,60*60)
+    local minutes = div(time,60)
+    local seconds = mod(time,60)
+    return days, hours, minutes, seconds
+end
+
+--
+
+local spacing = whitespace^0
+local apply   = P("->")
+local method  = C((1-apply)^1)
+local token   = lbrace * C((1-rbrace)^1) * rbrace + C(anything^1)
+
+local pattern = spacing * (method * spacing * apply + Carg(1)) * spacing * token
+
+function utilities.parsers.splitmethod(str,default)
+    if str then
+        return lpegmatch(pattern,str,1,default or false)
+    else
+        return default or false, ""
+    end
+end
+
+-- print(utilities.parsers.splitmethod(" foo -> {bar} "))
+-- print(utilities.parsers.splitmethod("foo->{bar}"))
+-- print(utilities.parsers.splitmethod("foo->bar"))
+-- print(utilities.parsers.splitmethod("foo"))
+-- print(utilities.parsers.splitmethod("{foo}"))
+-- print(utilities.parsers.splitmethod())
