@@ -100,3 +100,7 @@ fonts.names.resolvespec = fonts.names.resolve -- only supported in mkiv
 function fonts.names.getfilename(askedname,suffix)  -- only supported in mkiv
     return ""
 end
+
+function fonts.names.ignoredfile(filename) -- only supported in mkiv
+    return false -- will be overloaded
+end
